@@ -22,11 +22,10 @@ import widgets
 import soundcard, mpd
 import dataparser
 
-import const, utils, string
+import const, utils
 import os
 import sys
 import signal
-import types
 import cfg
 
 class Teacher(cfg.ConfigUtils):
@@ -36,8 +35,8 @@ class Teacher(cfg.ConfigUtils):
         self.q_status = const.QSTATUS_NO
         self.m_statistics = None
         self.m_timeout_handle = None
-        for k in config.keys():
-            self.set_string(k, dict[k])
+        for key, val in config.iteritems():
+            self.set_string(key, val)
     def get_instrument_config(self, num):
         if self.get_bool('override_default_instrument'):
             instr_low = self.get_int('lowest_instrument')
@@ -58,7 +57,7 @@ class Teacher(cfg.ConfigUtils):
                 instr_middle, instr_middle_vel, instr_high, instr_high_vel
     def maybe_auto_new_question(self):
         if self.get_bool('new_question_automatically'):
-            if self.m_timeout_handle == None:
+            if self.m_timeout_handle is None:
                 def remove_timeout(self=self):
                     self.m_timeout_handle = None
                     self.g_view.new_question()
@@ -70,7 +69,7 @@ class Teacher(cfg.ConfigUtils):
         self.q_status = const.QSTATUS_NO
         soundcard.synth.stop()
     def configure_exercise(self, collection, lessonfile, D):
-        for k in D.keys():
+        for k in D:
             self.set_string(k, D[k])
 
 class MelodicIntervalTeacher(Teacher):
@@ -88,8 +87,8 @@ class MelodicIntervalTeacher(Teacher):
         self.m_tonika = None
         self.m_question = []
     def new_question(self, L, H):
-        assert type(L) in types.StringTypes
-        assert type(H) in types.StringTypes
+        assert isinstance(L, basestring)
+        assert isinstance(H, basestring)
         if self.get_list('ask_for_intervals_0') == []:
             return self.ERR_CONFIGURE
         L, H = utils.adjust_low_high_to_irange(L, H,
@@ -226,18 +225,19 @@ You really should reastart the program now.""")
             run = self.m_app.m_ui.dialog_yesno(_("The lessonfile contain potentially dangerous code because the lesson file set the variable \"musicformat\" to \"cmdline\". Run anyway?"))
             if not run:
                 self.m_P = None
-    def configure_exercise(self, lessoncollection, lessonfile, dict):
-    	"""This function should not be called by the constructors.
-	abstract.Teacher and abstract.LessonbasedTeacher does the same job,
-	but in correct order.
-	"""
+    def configure_exercise(self, lessoncollection, lessonfile, config):
+        """
+        This function should not be called by the constructors.
+        abstract.Teacher and abstract.LessonbasedTeacher does the same job,
+        but in correct order.
+        """
         if lessoncollection or lessonfile:
             if lessoncollection:
                 self.set_string('lessoncollection', lessoncollection)
             if lessonfile:
                 self.set_string('lessonfile', lessonfile)
-        for k in dict.keys():
-            self.set_string(k, dict[k])
+        for key, val in config.iteritems():
+            self.set_string(key, val)
 
 class Gui(gtk.VBox, cfg.ConfigUtils):
     """Important members:
@@ -249,7 +249,6 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
     def __init__(self, teacher, window, no_notebook=0):
         gtk.VBox.__init__(self)
         cfg.ConfigUtils.__init__(self, teacher.m_exname)
-        #cfg.ConfigUtils.__dict__['__init__'](self, teacher.m_exname)
         self.m_key_bindings = {}
         self.m_t = teacher
         self.g_win = window
@@ -261,11 +260,11 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
 
         self.practise_box = gtk.VBox()
         self.practise_box.show()
-        vbox.pack_start(self.practise_box)
+        vbox.pack_start(self.practise_box, False)
 
         self.action_area = gtk.HBox()
         self.action_area.show()
-        vbox.pack_end(self.action_area, False)
+        vbox.pack_start(self.action_area, False)
 
         self.config_box = gtk.VBox()
         self.config_box.set_border_width(gnome.ui.PAD)
@@ -289,9 +288,9 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
     def on_end_practise(self):
         print "dummy abstract.Gui.on_end_practise", self.m_exname
     def on_key_press_event(self, widget, event):
-        if (self.g_notebook == None or self.g_notebook.get_current_page() == 0) \
+        if (self.g_notebook is None or self.g_notebook.get_current_page() == 0) \
            and event.type == gtk.gdk.KEY_PRESS:
-            for s in self.m_key_bindings.keys():
+            for s in self.m_key_bindings:
                 if self.keymatch(event, s):
                     self.m_key_bindings[s]()
                     return 1
@@ -427,7 +426,7 @@ class LessonbasedGui(Gui):
         button.connect('clicked', self.run_random_transpose_dialog)
         hbox.pack_start(button)
     def run_random_transpose_dialog(self, widget):
-        dlg = RandomTransposeDialog(self.m_t, self.g_win)
+        dlg = widgets.RandomTransposeDialog(self.m_t.m_P.header.random_transpose, self.g_win)
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             self.m_t.m_P.header.random_transpose = dlg.get_value()
@@ -440,97 +439,4 @@ class LessonbasedGui(Gui):
         self.config_box.pack_start(self.g_lessonfile_selector)
 
 
-class RandomTransposeDialog(gtk.Dialog):
-    def __init__(self, teacher, parent):
-        gtk.Dialog.__init__(self, _("Set transposition"), parent, 0, 
-           (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-            gtk.STOCK_OK, gtk.RESPONSE_OK,
-            ))
-        self.m_t = teacher
-        dlg_vbox = gu.hig_dlg_vbox()
-        self.vbox.pack_start(dlg_vbox)
-        xbox, vbox = gu.hig_category_vbox(_("Select how to do random transposition"))
-        dlg_vbox.pack_start(xbox)
-        label = gtk.Label(_("""You can read about the different types of transposition in the lesson file documentation available on the Help menu."""))
-        label.set_line_wrap(True)
-        vbox.pack_start(label)
-        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        self.m_buttons = {}
-        self.m_spins = {}
-        self.m_buttons['no'] = b1 = gtk.RadioButton(None, _("No"))
-        self.m_spins['no'] = []
-        vbox.pack_start(b1)
-        self.m_buttons['yes'] = b2 = gtk.RadioButton(b1, _("Yes"))
-        self.m_spins['yes'] = []
-        vbox.pack_start(b2)
-
-        self.m_buttons['accidentals'] = gtk.RadioButton(b1, "accidentals")
-        hbox = gu.bHBox(vbox)
-        hbox.pack_start(self.m_buttons['accidentals'])
-        self.m_spins['accidentals'] = [
-            gtk.SpinButton(gtk.Adjustment(0, -7, 7, 1, 1, 1), 0.0, 0),
-            gtk.SpinButton(gtk.Adjustment(0, -7, 7, 1, 1, 1), 0.0, 0)]
-        hbox.pack_start(self.m_spins['accidentals'][0])
-        hbox.pack_start(self.m_spins['accidentals'][1])
-        gu.SpinButtonRangeController(self.m_spins['accidentals'][0],
-                                     self.m_spins['accidentals'][1],
-                                     -7, 7)
-
-        self.m_buttons['key'] = gtk.RadioButton(b1, "key")
-        hbox = gu.bHBox(vbox)
-        hbox.pack_start(self.m_buttons['key'])
-        self.m_spins['key'] = [
-            gtk.SpinButton(gtk.Adjustment(0, -10, 10, 1, 1, 1), 0.0, 0),
-            gtk.SpinButton(gtk.Adjustment(0, -10, 10, 1, 1, 1), 0.0, 0)]
-        hbox.pack_start(self.m_spins['key'][0])
-        hbox.pack_start(self.m_spins['key'][1])
-        gu.SpinButtonRangeController(self.m_spins['key'][0],
-                                     self.m_spins['key'][1],
-                                     -10, 10)
-
-        self.m_buttons['semitones'] = gtk.RadioButton(b1, "semitones")
-        hbox = gu.bHBox(vbox)
-        hbox.pack_start(self.m_buttons['semitones'])
-        self.m_spins['semitones'] = [
-            gtk.SpinButton(gtk.Adjustment(0, -100, 100, 1, 1, 1), 0.0, 0),
-            gtk.SpinButton(gtk.Adjustment(0, -100, 100, 1, 1, 1), 0.0, 0)]
-        hbox.pack_start(self.m_spins['semitones'][0])
-        hbox.pack_start(self.m_spins['semitones'][1])
-        gu.SpinButtonRangeController(self.m_spins['semitones'][0],
-                                     self.m_spins['semitones'][1],
-                                     -100, 100)
-        for n, w in self.m_buttons.items():
-            sizegroup.add_widget(w)
-            for ww in self.m_spins[n]:
-                ww.set_sensitive(False)
-            w.connect('clicked', self.on_spins_clicked, n)
-        if self.m_t.m_P.header.random_transpose == False:
-            k = 'no'
-        elif self.m_t.m_P.header.random_transpose == True:
-            k = 'yes'
-        else:
-            k = self.m_t.m_P.header.random_transpose[0]
-        self.m_buttons[k].set_active(True)
-        if k in ('accidentals', 'key', 'semitones'):
-            t, v1, v2 = self.m_t.m_P.header.random_transpose
-            # FIXME Because of the RangeController, we have to do this
-            # twice to be sure both values are set properly.
-            self.m_spins[t][1].set_value(v2)
-            self.m_spins[t][0].set_value(v1)
-            self.m_spins[t][1].set_value(v2)
-            self.m_spins[t][0].set_value(v1)
-        self.show_all()
-    def on_spins_clicked(self, w, n):
-        for w in self.m_spins[n]:
-            w.set_sensitive(self.m_buttons[n].get_active())
-    def get_value(self):
-        for n, btn in self.m_buttons.items():
-            if btn.get_active():
-                s = n
-        if s == 'yes':
-            return True
-        elif s == 'no':
-            return False
-        return [s, self.m_spins[s][0].get_value_as_int(), self.m_spins[s][1].get_value_as_int()]
-
 
