@@ -1,5 +1,5 @@
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,16 +15,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
+
 MAJOR_VERSION = 2
 MINOR_VERSION = 1
-PATCH_LEVEL= 4
+PATCH_LEVEL = 10
 VERSION_STRING = '%s.%s.%s' % (MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL)
-if "yes" == "yes":
-   WITH_GNOME = 1
+if "no" == "yes":
+    WITH_GNOME = 1
 else:
-   WITH_GNOME = 0
-WITH_GTKHTML = "yes" == 'yes'
+    WITH_GNOME = 0
+WITH_GTKHTML = "no" == 'yes'
 prefix = "/usr/local"
-sysconfdir = "${prefix}/etc"
+datadir = os.path.join(prefix, "share", "solfege", VERSION_STRING)
 HAVE_LINUX_AWE_VOICE_H = "yes" == "yes"
 ENABLE_TUNER = "no" == "yes"
