@@ -18,7 +18,7 @@
 __exercise_data__ = {
     'exercise_name': 'identify-bpm',
     'menu_path': '%s/%s' % (_("_Rhythm"), _("Bpm")),
-    'icon': 'graphics/identify-bpm24.png',
+    'stock-icon': 'solfege-identify-bpm',
     'toolbar': 12,
 }
 import exercise_setup
@@ -58,7 +58,7 @@ class Teacher(abstract.Teacher):
         self.set_string('active_bpms', str(v))
     def get_possible_bpms(self):
         v = []
-        for x in self.m_practise_these.keys():
+        for x in self.m_practise_these:
             if self.m_practise_these[x]:
                 v.append(x)
         return v#UGH
@@ -104,6 +104,7 @@ class Teacher(abstract.Teacher):
             self.m_app.play_sad_sound()
     def end_practise(self):
         self.stop_question()
+        self.q_status = const.QSTATUS_NO
     def give_up(self):
         self.stop_question()
         print "FIXME not saving statistics"
@@ -194,6 +195,7 @@ class Gui(abstract.Gui):
             self.g_flashbar.flash(_("Wrong"))
         self.g_statview.update()
     def on_start_practise(self):
+        self.g_flashbar.flash(_("Click 'New tempo' to begin."))
         self.g_new.grab_focus()
     def on_end_practise(self):
         self.m_t.end_practise()
