@@ -17,8 +17,8 @@
 
 __exercise_data__ = {
     'exercise_name': 'id-tone',
-    'menu_path': '%s/%s' % (_("_Misc exercises"), _("_Id_entify tone")),
-    'icon': 'graphics/idtone24.png',
+    'menu_path': '%s/%s' % (_("_Misc exercises"), _("Id_entify tone")),
+    'stock-icon': 'solfege-id-tone',
     'toolbar': 8,
 }
 import exercise_setup
@@ -152,22 +152,22 @@ right now. But I can answer questions if needed.""")))
         frame.add(table)
         for x, n in [(1, 'cis'), (3, 'dis'), (7, 'fis'),
                      (9, 'gis'), (11, 'ais')]:
-             label = gtk.Label(mpd.MusicalPitch.new_from_notename(n).get_user_notename())
-             label.set_name("Heading2")
-             label.set_alignment(0.2, 1.0)
-             table.attach(label, x, x+2, 0, 1, xoptions=gtk.FILL)
-             b = gu.nSpinButton(self.m_exname, n+"_weight",
+            label = gtk.Label(mpd.MusicalPitch.new_from_notename(n).get_user_notename())
+            label.set_name("Heading2")
+            label.set_alignment(0.2, 1.0)
+            table.attach(label, x, x+2, 0, 1, xoptions=gtk.FILL)
+            b = gu.nSpinButton(self.m_exname, n+"_weight",
                       gtk.Adjustment(1, 0, 1000, 1, 10), digits=0)
-             table.attach(b, x, x+2, 1, 2, xoptions=gtk.FILL)
+            table.attach(b, x, x+2, 1, 2, xoptions=gtk.FILL)
         for x, n in [(0, 'c'), (2, 'd'), (4, 'e'), (6, 'f'),
                       (8, 'g'), (10, 'a'), (12, 'b')]:
-             label = gtk.Label(mpd.MusicalPitch.new_from_notename(n).get_user_notename())
-             label.set_name("Heading2")
-             label.set_alignment(0.35, 1.0)
-             table.attach(label, x, x+2, 2, 3, xoptions=gtk.FILL)
-             b = gu.nSpinButton(self.m_exname, n+"_weight",
+            label = gtk.Label(mpd.MusicalPitch.new_from_notename(n).get_user_notename())
+            label.set_name("Heading2")
+            label.set_alignment(0.35, 1.0)
+            table.attach(label, x, x+2, 2, 3, xoptions=gtk.FILL)
+            b = gu.nSpinButton(self.m_exname, n+"_weight",
                    gtk.Adjustment(1, 0, 1000, 1, 10), digits=0)
-             table.attach(b, x, x+2, 3, 4, xoptions=gtk.FILL)
+            table.attach(b, x, x+2, 3, 4, xoptions=gtk.FILL)
 
         hbox = gu.bHBox(self.config_box, False)
         hbox.pack_start(gtk.Label(_("Octave:")), False, padding=4)
