@@ -1,5 +1,5 @@
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -107,17 +107,16 @@ predef = {'dictation': 'dictation',
           'semitones': 'semitones',
           'tempo': (60, 4)}
 
-class WavPlayerFunctionClass(object):
-    def __init__(self, args):
-        self.m_args = args
-    def __str__(self):
-        return cfg.get_string("sound/wav_player") + " " + self.m_args
-
 lessonfile_functions = {
         '_': dataparser.dataparser_i18n_func,
         '_i': _i, #FIXME
-        'play_wav': lambda f: WavPlayerFunctionClass(f),
+        'play_wav': lambda f: Music(cfg.get_string("sound/wav_player") + " " + f, "cmdline"),
         'music': lambda m, t='normal': Music(m, t),
+        'chord': lambda m: Music(m, 'chord'),
+        'satb': lambda m: Music(m, 'satb'),
+        'voice': lambda m: Music(m, 'voice'),
+        'rvoice': lambda m: Music(m, 'rvoice'),
+        'cmdline': lambda m: Music(m, 'cmdline'),
 }
 
 class _Header:
@@ -177,7 +176,8 @@ class LessonfileCommon(object):
             question['m_lessonfile'] = self
     def select_random_question(self):
         """
-        Select a new question by random.
+        Select a new question by random. It will use the music in the
+        lesson file question variable 'music' when selecting transposition.
         """
         count = 0
         available_question_idx = []
@@ -314,10 +314,40 @@ class LessonfileCommon(object):
             return r"\staff\transpose %s{ %s }" \
               % (self.m_transpose.get_octave_notename(), question[varname].m_musicdata)
         elif question[varname].m_musictype == 'rvoice':
-            ns = question[varname].m_musicdata.split()[0]
-            rel_note = re.match("[a-zA-Z]+[',]*[\d]*\.*", ns).group()
-            return r"\staff\transpose %s\relative c'{ %s }" \
-              % (self.m_transpose.get_octave_notename(), question[varname].m_musicdata)
+            def find_first_note(v):
+                """arg v is a list of the musicdata splittet by whitespace.
+                Return the index of the first note.
+                """
+                #FIXME tidy up a little
+                i = 0
+                while 1:
+                    n = re.match("(\<?)[a-zA-Z]+[',]*", v[i])
+                    if n:
+                        break
+                    if v[i] == r"\clef":
+                        i += 2
+                    elif v[i] == r"\key":
+                        i += 3
+                    elif v[i] == r"\time":
+                        i += 2
+                    else:
+                        i += 1
+                    if i >= len(v):
+                        return -1
+                return i
+            v = question[varname].m_musicdata.split()
+            i = find_first_note(v)
+            if i == -1:
+                return r"\staff\transpose %s{ %s }"\
+                   % (self.m_transpose.get_octave_notename(), " ".join(v))
+            m = re.match("(?P<pre>\<?)(?P<note>[a-zA-Z]+[',]*)", v[i])
+            rel_note = m.group('note')
+            nn = mpd.MusicalPitch.new_from_notename(rel_note)
+            aa = m.group('pre') + nn.get_octave_notename()
+            v[i] = m.group('pre') + nn.get_notename() + v[i][len(aa)::]
+            return r"\staff\transpose %s\relative %s{ %s }" \
+              % (self.m_transpose.get_octave_notename(), 
+                 rel_note, " ".join(v))
         elif question[varname].m_musictype == 'cmdline':
             return str(question[varname])
         else:
@@ -429,10 +459,10 @@ class LessonfileCommon(object):
     def play_question_slowly(self, question=None):
         if not question:
             question = self.get_question()
-        assert question['music'].m_musictype == 'normal'
+        #assert question['music'].m_musictype == 'normal'
+        #FIXME what music types should be handled differently
         self.play_question_musicformat_normal_slowly(question)
     def play_question_musicformat_normal_slowly(self, question):
-        assert question['music'].m_musictype == 'normal'
         instrument = self.prepare_instrument_list(question)
         bpm = self.get_tempo()
         bpm = (bpm[0]/2, bpm[1])
