@@ -1,5 +1,5 @@
-# GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
+# GNU Solfege - ear training for GNOME
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,19 +15,28 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import __builtin__
+__builtin__.app_running = True
+__builtin__.__exdata__ = []
+
 import sys, os, time, random
-# Hack to make module testing work.
-__builtins__['app_running'] = 'solfege'
+
+if sys.version_info < (2, 3):
+    print "********************************************************************"
+    print "Solfege need Python 2.3 or newer. The configure script told you so!"
+    print "********************************************************************"
+    sys.exit(-1)
 
 # i18n should be imported very early in program init because it setup
 # the _ and _i functions for the whole program.
 import i18n
+i18n.setup()
 
 import configureoutput
 
 def do_some_cmdline_options(argv):
-        if '-h' in argv or '--help' in argv:
-            print """
+    if '-h' in argv or '--help' in argv:
+        print """
 GNU Solfege %s
 Type "man solfege" for more details.
 Options:
@@ -38,21 +47,21 @@ Options:
   -h, --help             Display this message
   --gtk-help             Display the usual gtk/gnome commandline options.
     """ % configureoutput.VERSION_STRING
-            sys.exit()
+        sys.exit()
 
-        if '-v' in argv or '--version' in argv:
-            print """GNU Solfege %s
+    if '-v' in argv or '--version' in argv:
+        print """GNU Solfege %s
 This is free software. It is covered by the GNU General Public License,
 and you are welcome to change it and/or distribute copies of it under
 certain conditions. Invoke as `solfege --warranty` for more information.
 
-Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen <tca@gnu.org>
-            """ % configureoutput.VERSION_STRING
-            sys.exit()
+Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen <tca@gnu.org>
+        """ % configureoutput.VERSION_STRING
+        sys.exit()
 
-        if '-w' in argv or '--warranty' in argv:
-            print """GNU Solfege %s
-Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen <tca@gnu.org>
+    if '-w' in argv or '--warranty' in argv:
+        print """GNU Solfege %s
+Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen <tca@gnu.org>
 
     This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License version 2
@@ -68,7 +77,7 @@ GNU General Public License along with this program; if not, write to
 the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 USA.
         """ % configureoutput.VERSION_STRING
-            sys.exit()
+        sys.exit()
 
 
 do_some_cmdline_options(sys.argv)
@@ -80,11 +89,12 @@ if '--gtk-help' in sys.argv:
 # silent, GNOME, be silent!
 sys.argv.append('--disable-sound')
 
-try:
-    import pygtk
+import pygtk
+# this is needed for py2exe
+if sys.platform == 'win32':
+    os.environ['PATH'] += ";lib;bin;"
+else:
     pygtk.require("2.0")
-except:
-    pass
 
 if not configureoutput.WITH_GNOME:
     sys.path.insert(0, "gnomeemu")
@@ -105,7 +115,7 @@ class SplashWin(gtk.Window):
     def __init__(self):
         gtk.Window.__init__(self, gtk.WINDOW_POPUP)
         self.set_position(gtk.WIN_POS_CENTER)
-        self.set_resizable(gtk.TRUE)
+        self.set_resizable(True)
         frame = gtk.Frame()
         frame.set_shadow_type(gtk.SHADOW_OUT)
         self.add(frame)
@@ -123,14 +133,14 @@ class SplashWin(gtk.Window):
     def show_progress(self, txt):
         self.g_infolabel.set_text(txt)
         while gtk.events_pending():
-            gtk.mainiteration(0)
+            gtk.main_iteration(0)
 
 if not filter(lambda a: a in sys.argv, ['--no-splash', '--usage', '-?', '--gtk-help']):
     splash_win = SplashWin()
     time.sleep(0.1)
     gtk.gdk.flush()
     while gtk.events_pending():
-        gtk.mainiteration(0)
+        gtk.main_iteration(0)
 else:
     splash_win = None
     if '--no-splash' in sys.argv:
@@ -149,9 +159,9 @@ if splash_win:
 
 import gnome, gnome.ui
 # FIXME we set this because it is not yet added to PyGNOME2
-gnome.ui.PAD=8
-gnome.ui.PAD_SMALL=4
-gnome.ui.PAD_BIG=12
+gnome.ui.PAD = 8
+gnome.ui.PAD_SMALL = 4
+gnome.ui.PAD_BIG = 12
 
 if splash_win:
     splash_win.show_progress("importing application modules")
@@ -162,18 +172,39 @@ import tracebackwindow
 
 sys.stderr = tracebackwindow.TracebackWindow()
 
-import const, string
+import const
 import app
 from htmlwidget import HtmlWidget
 import utils, history
 from configwindow import ConfigWindow
-import mpd, mpd.musicdisplayer, gu, cfg
 import soundcard
+import mpd, mpd.musicdisplayer, gu, cfg
+import stock
+
+
+# The order the modules are imported affect the ordering in the menus.
+import melodicinterval, chord, rhythm
+import chordvoicing, harmonicinterval, \
+       singinterval, idbyname, dictation, twelvetone, \
+       idtone, compareintervals, singchord, identifybpm, \
+       harmonicprogressiondictation, singanswer,\
+       polyrhythm, intervals
+
+import exercise_setup
+exercise_setup.register_exercise({
+    'exercise_name': 'id-by-name',
+    'exercise_config': 'solfege/scales',
+    'menu_path': '%s/%s' % (_("_Misc exercises"), _("_Scales")),
+    'toolbar': 4.5,
+    'stock-icon': 'solfege-identify-scale',
+})
+exercise_setup.register_exercise({
+    'exercise_name': 'id-by-name',
+    'exercise_config': 'solfege/scales-thirds',
+    'menu_path': '%s/%s' % (_("_Misc exercises"), _("Scales in t_hirds")),
+    'stock-icon': 'solfege-identify-scale',
+})
 
-import chord, chordvoicing, harmonicinterval, melodicinterval, \
-       singinterval, identifyscale, idbyname, dictation, twelvetone, \
-       idtone, compareintervals, singchord, rhythm, identifybpm, \
-       harmonicprogressiondictation
 
 class DocViewer(gtk.VBox):
     def __init__(self, activate_cb):
@@ -181,13 +212,13 @@ class DocViewer(gtk.VBox):
         self.m_htmlwidget = HtmlWidget(activate_cb, self.on_anchor_track)
         self.m_htmlwidget.show_all()
         self.pack_start(self.m_htmlwidget)
-        self.set_size_request(300, 300)
+        self.set_size_request(500, 300)
         #self.read_docfile = self.m_htmlwidget.read_docfile
         self.get_vadjustment = self.m_htmlwidget.get_vadjustment
         self.on_key_press_event = self.m_htmlwidget.on_key_press_event
         self.m_statusbar = gtk.Statusbar()
         self.m_statusbar.show()
-        self.pack_start(self.m_statusbar, gtk.FALSE)
+        self.pack_start(self.m_statusbar, False)
         self.source = self.m_htmlwidget.source
         self.grab_focus = self.m_htmlwidget.grab_focus
     def read_docfile(self, fn, anchor):
@@ -206,7 +237,7 @@ class DocViewer(gtk.VBox):
     def on_anchor_track(self, url):
         if url:
             # remove newlines in url because it make the window resize
-            s = string.replace(url, "\n", "")
+            s = url.replace("\n", "")
             self.m_statusbar.pop(1)
             self.m_statusbar.push(1, s)
         else:
@@ -217,7 +248,7 @@ class MusicViewerWindow(gtk.Dialog):
     def __init__(self, app):
         gtk.Dialog.__init__(self)
         self.m_app = app
-        self.set_size_request(400, 200)
+        self.set_default_size(400, 200)
         self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
         self.vbox.pack_start(self.g_music_displayer)
         b = gu.bButton(self.action_area, _("Close"), self.m_app.close_musicviewer)
@@ -228,13 +259,13 @@ class MusicViewerWindow(gtk.Dialog):
         fontsize = cfg.get_int('config/feta_font_size=20')
         self.g_music_displayer.display(music, fontsize)
 
-
 class MainWin(gnome.ui.App, cfg.ConfigUtils):
     def __init__(self, argv):
         gnome.ui.App.__init__(self, 'solfege19', 'GNU Solfege')
-        im = gtk.Image()
-        im.set_from_file(os.path.join("graphics", "solfege.png"))
-        self.set_icon(im.get_pixbuf())
+        self.icons = stock.SolfegeIconFactory(self, configureoutput.datadir)
+        self.set_default_size(400, 400)
+        pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
+        self.set_icon(pixbuf)
         cfg.ConfigUtils.__dict__['__init__'](self, 'mainwin')
         self.set_resizable(self.get_bool('gui/mainwin_user_resizeable'))
         self.connect('destroy', self.quit_program)
@@ -256,26 +287,48 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.setup_toolbar()
         self.setup_menubar()
         self.m_app.handle_href('welcome.html')
-    def display_error_message(self, msg):
+    def dialog_yesno(self, text):
+        """Return True if the answer is yes, False if the answer is no.
+        """
+        m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION,
+                gtk.BUTTONS_YES_NO, text)
+        ret = m.run()
+        m.destroy()
+        return ret == gtk.RESPONSE_YES
+    def display_exception_message(self, exception, err_file=None):
+        """Call this function only inside an except clause."""
+        if err_file:
+            msg = _("Exception catched in %s:%i\n") % (err_file, sys.exc_info()[2].tb_lineno) + "\n"
+        else:
+            msg = ""
+        msg += str(exception)
+        m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
+                              gtk.BUTTONS_CLOSE, msg)
+        m.set_title(sys.exc_info()[0].__name__)
+        m.run()
+        m.destroy()
+    def display_error_message(self, msg, err_pos=None):
+        if err_pos:
+            msg = _("Error found in %s:\n") % err_pos + "\n" + msg
         m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_CLOSE, msg)
         m.run()
         m.destroy()
-    def display_question_music_error_message(self, idx, lessonfile, exception):
-        self.display_error_message(_("Failed to parse the music for question number %(idx)i in '%(lf)s':\n%(ex)s")
-          % {'idx': idx, 'lf': lessonfile, 'ex': exception})
+    def display_question_music_error_message(self, idx, lessonfile, exception, err_file=None):
+        """Call this function only inside an except clause."""
+        msg = _("Failed to parse the music for question number %(idx)i in '%(lf)s':\n%(ex)s") % {'idx': idx, 'lf': lessonfile, 'ex': exception}
+        self.display_exception_message(msg, err_file)
     def show_about_window(self, widget):
         #if not self.g_about_window:
         if 1:
-            im = gtk.Image()
-            im.set_from_file(os.path.join("graphics", "solfege.png"))
             trans = _("SOLFEGETRANSLATORS")
             if trans == 'SOLFEGETRANSLATORS':
                 trans = ""
+            pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
             self.g_about_window = gnome.ui.About("GNU Solfege",
                   configureoutput.VERSION_STRING,
-                  "Copyright (C) 2000, 2001, 2002, 2003 Tom Cato Amundsen",
-                  _("GPL'ed eartraining.") + "\n" + "http://www.solfege.org",
+                  "Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen",
+                  _("GPL'ed ear training.") + "\n" + "http://www.solfege.org",
                   ["Tom Cato Amundsen",
                   'Oliver Vollmer %s' % _("(toolbar icons)"),
                   'Giovanni Chierico %s' % _("(some lessonfiles)"),
@@ -283,8 +336,7 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
                   'Joe Lee %s' % _("(sound code for the MS Windows port)"),
                   ],
                   ["Tom Cato Amundsen"],
-                  trans,
-                  im.get_pixbuf())
+                  trans, pixbuf)
             #self.g_about_window.connect('destroy', self.hide_about_window)
             #self.g_about_window.show()
             #FIXME this sucks.
@@ -317,11 +369,11 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.set_title("Solfege - " + self.box_dict[exname].get_pretty_name())
         self.m_history.add(href)
     def _show_docviewer(self):
-        if not self.box_dict.has_key('docviewer'):
+        if 'docviewer' not in self.box_dict:
             self.box_dict['docviewer'] = DocViewer(self.m_app.handle_href)
             self.main_box.pack_start(self.box_dict['docviewer'])
         if self.m_viewer and (self.m_viewer != 'docviewer'):
-                self.box_dict[self.m_viewer].hide()
+            self.box_dict[self.m_viewer].hide()
         self.m_viewer = 'docviewer'
         self.box_dict['docviewer'].show()
     def display_docfile(self, fn, anchor):
@@ -359,13 +411,13 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.m_app.handle_href(self.m_history.get_current()[0])
         self.m_history.unlock()
     def initialise_exercise(self, teacher):
-        assert not self.box_dict.has_key(teacher.m_exname)
+        assert teacher.m_exname not in self.box_dict
         n = utils.exercise_name_to_module_name(teacher.m_exname)
         self.box_dict[teacher.m_exname] = globals()[n].Gui(teacher, self)
         self.main_box.pack_start(self.box_dict[teacher.m_exname])
     def on_key_press_event(self, widget, event):
         if event.type == gtk.gdk.KEY_PRESS:
-            for s in self.m_key_bindings.keys():
+            for s in self.m_key_bindings:
                 if (event.state & (gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK), event.keyval) \
                          == utils.parse_key_string(self.get_string(s)):
                     self.m_key_bindings[s]()
@@ -390,35 +442,86 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             menubar.add(menuitem)
             menuitem.set_submenu(func(ag))
         self.g_menubar.show_all()
+    def create_submenu(self, data, ag):
+        """FIXME rename
+        """
+        menu = gtk.Menu()
+        for d in data:
+            menuitem = gtk.ImageMenuItem(d['menu_path'].split('/')[1])
+            if 'icon' in d:
+                im = gtk.Image()
+                im.set_from_file(d['icon'])
+                im.set_from_pixbuf(im.get_pixbuf().scale_simple(16, 16, gtk.gdk.INTERP_TILES))
+                menuitem.set_image(im)
+            elif 'stock-icon' in d:
+                im = gtk.Image()
+                im.set_from_stock(d['stock-icon'], gtk.ICON_SIZE_MENU)
+                menuitem.set_image(im)
+            if 'exercise_config' in d:
+                s = 'solfege:practise/%s/%s' % (d['exercise_name'], d['exercise_config'])
+            else:
+                s = 'solfege:practise/%s' % d['exercise_name']
+            menuitem.connect('activate', lambda o, self=self, s=s: self.m_app.handle_href(s))
+            if 'accel_key' in d:
+                menuitem.add_accelerator('activate', ag, d['accel_key'], d['accel_mods'], gtk.ACCEL_VISIBLE)
+            menu.add(menuitem)
+        return menu
+    def create_ex_menu(self, exlist, ag):
+        """
+        Each item in exlist:
+        (MENUTEXT, URI, graphics/FILENAME.PNG)
+        The icon is optional.
+        """
+        menu = gtk.Menu()
+        for ex in exlist:
+            if ex[0] == ex[1] == 'SEP':
+                item = gtk.SeparatorMenuItem()
+            else:
+                item = gtk.ImageMenuItem(ex[0])
+                item.connect('activate', lambda o, url=ex[1]: self.m_app.handle_href(url))
+                if len(ex) > 2:
+                    im = gtk.Image()
+                    im.set_from_file(os.path.join('graphics', ex[2]))
+                    im.set_from_pixbuf(im.get_pixbuf().scale_simple(16, 16, gtk.gdk.INTERP_TILES))
+                    item.set_image(im)
+            menu.add(item)
+        return menu
+    def get_file_menu_data(self):
+        menus = {}
+        sections = []
+        for rec in __exdata__:
+            m, n = rec['menu_path'].split("/")
+            if m not in menus:
+                menus[m] = []
+                sections.append(m)
+            menus[m].append(rec)
+        return sections, menus
     def create_file_menu(self, ag):
         menu = gtk.Menu()
         if self.get_bool("gui/menus_have_tearoff"):
             item = gtk.TearoffMenuItem()
             menu.add(item)
-        for ex in const.exercises:
-            box = gtk.HBox()
-            box.set_spacing(4)
-            if const.exercisedata[ex][1]:
-                item = gtk.ImageMenuItem(const.exercisedata[ex][0])
-                im = gtk.Image()
-                im.set_from_file(os.path.join('graphics', const.exercisedata[ex][1]))
-                im.set_from_pixbuf(im.get_pixbuf().scale_simple(16, 16, gtk.gdk.INTERP_TILES))
-                item.set_image(im)
-            else:
-                item = gtk.MenuItem(const.exercisedata[ex][0])
-            if const.exercisedata[ex][2]:
-                item.add_accelerator('activate', ag, const.exercisedata[ex][2], const.exercisedata[ex][3], gtk.ACCEL_VISIBLE)
-            item.connect('activate', lambda o, self=self, ex=ex: self.m_app.handle_href('solfege:practise/'+ex))
-            menu.add(item)
+        sections, d = self.get_file_menu_data()
+        for k in sections:
+            menuitem = gtk.MenuItem(k)
+            menu.add(menuitem)
+            menuitem.set_submenu(self.create_submenu(d[k], ag))
+        ##
+        menuitem = gtk.MenuItem(_("_Theory"))
+        menu.add(menuitem)
+        menuitem.set_submenu(self.create_ex_menu(
+              ( ('_Scale names', 'theory-scales.html'),
+                ('_Intervals', 'theory-intervals.html'),
+               ), ag))
         item = gtk.MenuItem()
         item.add(gtk.HSeparator())
         menu.add(item)
         ##############
         # Preferences
-        menu.add(create_stock_menu_item('gtk-preferences', _("Preferences window"), self.open_preferences_window, ag, gtk.keysyms.F12, gtk.gdk.CONTROL_MASK))
+        menu.add(gu.create_stock_menu_item('gtk-preferences', _("Preferences window"), self.open_preferences_window, ag, gtk.keysyms.F12, gtk.gdk.CONTROL_MASK))
         #######
         # Exit
-        menu.add(create_stock_menu_item('gtk-quit', _("_Exit"), self.quit_program, ag, gtk.keysyms.Q, gtk.gdk.CONTROL_MASK))
+        menu.add(gu.create_stock_menu_item('gtk-quit', _("_Exit"), self.quit_program, ag, gtk.keysyms.Q, gtk.gdk.CONTROL_MASK))
         return menu
     def create_view_menu(self, ag):
         menu = gtk.Menu()
@@ -441,32 +544,20 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
           navitem.set_active(self.get_bool('gui/navbar_visible')))
         menu.add(navitem)
 
-        menu.add(gtk.SeparatorMenuItem())
-
-        item = gtk.CheckMenuItem(_("_User resizeable main window"))
-        item.connect('toggled', self.on_view_menu_user_resizeable_toggled)
-        item.set_active(self.get_bool('gui/mainwin_user_resizeable'))
-        self.add_watch('gui/mainwin_user_resizeable', lambda key:
-          item.set_active(self.get_bool('gui/mainwin_user_resizeable')))
-        menu.add(item)
-
         if configureoutput.WITH_GTKHTML:
             menu.add(gtk.SeparatorMenuItem())
-            menu.add(create_stock_menu_item('gtk-zoom-in', _("Help viewer zoom in"),
+            menu.add(gu.create_stock_menu_item('gtk-zoom-in', _("Help viewer zoom in"),
                lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in(),
                ag, gtk.keysyms.plus, gtk.gdk.CONTROL_MASK))
-            menu.add(create_stock_menu_item('gtk-zoom-out', _("Help viewer zoom out"),
+            menu.add(gu.create_stock_menu_item('gtk-zoom-out', _("Help viewer zoom out"),
                lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_out(),
                ag, gtk.keysyms.minus, gtk.gdk.CONTROL_MASK))
-            menu.add(create_stock_menu_item('gtk-zoom-100', _("Help viewer zoom reset"),
+            menu.add(gu.create_stock_menu_item('gtk-zoom-100', _("Help viewer zoom reset"),
                lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_reset(),
                ag, gtk.keysyms.equal, gtk.gdk.CONTROL_MASK))
         return menu
     def on_zoom_in(self, *v):
         self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in()
-    def on_view_menu_user_resizeable_toggled(self, widget):
-        self.set_bool('gui/mainwin_user_resizeable', widget.get_active())
-        self.set_resizable(widget.get_active())
     def on_view_menu_toolbar_visible_toggled(self, widget):
         """
         This function is called when the user selects the View/Toolbar menu
@@ -492,13 +583,13 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         if self.get_bool("gui/menus_have_tearoff"):
             item = gtk.TearoffMenuItem()
             menu.add(item)
-        item = create_stock_menu_item('gtk-help', _("_Help on current exercise"),
+        item = gu.create_stock_menu_item('gtk-help', _("_Help on current exercise"),
                             lambda o, self=self: self.m_app.please_help_me(),
                             ag, gtk.keysyms.F1, 0)
         menu.add(item)
         for txt, url in (
                      (_("_Welcome"), 'welcome.html'),
-                     (_("_All help files"), 'index.html'),
+                     (_("_All help files"), 'toc.html'),
                      (_("All installed _lesson files"), 'solfege:all-lessonfiles'),
                      ('SEP', 'SEP'),
                      (_("_Copyright notice"), 'copyright.html'),
@@ -517,8 +608,6 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         item.connect('activate', self.show_about_window)
         menu.add(item)
         return menu
-    def experiment(self, w=None):
-        pass
     def quit_program(self, w=None):
         self.m_app.quit_program()
         gtk.main_quit()
@@ -544,13 +633,28 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             self.get_dock_item_by_name("navbar").hide()
         # main toolbar
         self.g_toolbar =  gtk.Toolbar()
-        for ex in const.exercises:
+        exdict = {}
+        tblist_ordered = []
+        tblist_unordered = []
+        for e in __exdata__:
+            exdict[e['exercise_name']] = e
+            if 'toolbar' in e:
+                if isinstance(e['toolbar'], bool) and e['toolbar'] == True:
+                    tblist_unordered.append(e)
+                else:
+                    tblist_ordered.append(e)
+        tblist_ordered.sort(lambda a, b: cmp(a['toolbar'], b['toolbar']))
+        for ex in tblist_ordered + tblist_unordered:
             im = gtk.Image()
-            im.set_from_file(os.path.join('graphics', const.exercisedata[ex][1]))
-            n = string.replace(const.exercisedata[ex][0], '_', '')
-            button = self.g_toolbar.append_item(n, None, None, im, None, None)
-            button.connect('clicked',
-                  lambda button, ex=ex: self.m_app.handle_href("solfege:practice/"+ex))
+            if 'toolbar' not in ex:
+                continue
+            if ex.has_key('icon'):
+                im.set_from_file(ex['icon'])
+            else:
+                assert ex.has_key('stock-icon')
+                im.set_from_stock(ex['stock-icon'], gtk.ICON_SIZE_SMALL_TOOLBAR)
+            button = self.g_toolbar.append_item(ex['exercise_pretty_name'], ex['exercise_pretty_name'], None, im, None, None)
+            button.connect('clicked', lambda button, ex=ex: self.m_app.handle_href("solfege:practice/"+ex['exercise_name']+'/'+ex['exercise_config']))
         self.add_toolbar(self.g_toolbar, "toolbar", "normal", "top", 1, 0, 1)
         s = self.get_string('gui/toolbar_style')
         if not (s in ('icons', 'text', 'both')):
@@ -576,17 +680,6 @@ def create_menu_item(txt, callback, ag, accel_key, accel_mod):
     item.connect('activate', callback)
     return item
 
-def create_stock_menu_item(stock, txt, callback, ag, accel_key, accel_mod):
-    box = gtk.HBox()
-    box.set_spacing(gnome.ui.PAD_SMALL)
-    im = gtk.Image()
-    im.set_from_stock(stock, gtk.ICON_SIZE_MENU)
-    item = gtk.ImageMenuItem(txt)
-    item.set_image(im)
-    item.add_accelerator('activate', ag, accel_key, accel_mod, gtk.ACCEL_VISIBLE)
-    item.connect('activate', callback)
-    return item
-
 if splash_win:
     splash_win.show_progress("running gnome.program_init")
 
@@ -607,5 +700,5 @@ w.show()
 if splash_win:
     splash_win.destroy()
 w.post_constructor()
-gtk.mainloop()
+gtk.main()
 
