@@ -1,5 +1,5 @@
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 __exercise_data__ = {
     'exercise_name': 'rhythm',
     'menu_path': '%s/%s' % (_("_Rhythm"), _("Rh_ythm")),
-    'icon': 'graphics/rhythm24.png',
+    'stock-icon': 'solfege-rhythm',
     'toolbar': 11,
 }
 import exercise_setup
@@ -124,10 +124,9 @@ def create_rhythm_image(rhythm):
     rhythm : a string like 'c8 c8' or 'c8 c16 c16'
     The image returned is shown.
     """
-    fn = 'rhythm-%s' % ''.join(rhythm.split())
     im = gtk.Image()
-    im.set_from_file(os.path.join('graphics', fn)+'.png')
-    #im.set_from_pixbuf(im.get_pixbuf().scale_simple(24, 24, gtk.gdk.INTERP_BILINEAR))
+    im.set_from_stock('solfege-rhythm-%s' % ''.join(rhythm.split()),
+                      gtk.ICON_SIZE_LARGE_TOOLBAR)
     im.show()
     return im
 
@@ -178,12 +177,6 @@ class RhythmViewer(gtk.Frame):
         self.m_data = self.m_data[:n]
         for x in range(n, self.m_num_beats):
             self.g_box.pack_start(create_png_image('holder'), False)
-    def add_elem(self, fn):
-        #UGH remove/rename/whatever?
-        im = gtk.Image()
-        im.set_from_file('graphics/%s' % fn)
-        im.show()
-        self.g_box.pack_start(im, False)
     def add_rhythm_element(self, i):
         assert len(self.m_data) <= self.m_num_beats
         if len(self.g_box.get_children()) >= self.m_num_beats:
@@ -201,6 +194,10 @@ class RhythmViewer(gtk.Frame):
             self.g_box.pack_start(create_png_image('holder'), False)
             del self.m_data[-1]
     def mark_wrong(self, idx):
+        """
+        Mark the rhythm elements that was wrong by putting the content of
+        graphics/rhythm-wrong.png (normally a red line) under the element.
+        """
         box = gtk.VBox()
         r = self.g_box.get_children()[idx]
         self.g_box.get_children()[idx].reparent(box)
@@ -215,13 +212,19 @@ class RhythmViewer(gtk.Frame):
         self.g_face = gtk.EventBox()
         self.g_face.connect('button_press_event', self.on_sadface_event)
         self.g_face.show()
-        self.g_face.add(create_png_image('sadface-32x32'))
+        im = gtk.Image()
+        im.set_from_stock('solfege-sadface', gtk.ICON_SIZE_LARGE_TOOLBAR)
+        im.show()
+        self.g_face.add(im)
         self.g_box.pack_start(self.g_face, False)
     def happy_face(self):
         self.g_face = gtk.EventBox()
         self.g_face.connect('button_press_event', self.on_happyface_event)
         self.g_face.show()
-        self.g_face.add(create_png_image("happyface-32x32"))
+        im = gtk.Image()
+        im.set_from_stock('solfege-happyface', gtk.ICON_SIZE_LARGE_TOOLBAR)
+        im.show()
+        self.g_face.add(im)
         self.g_box.pack_start(self.g_face, False)
     def on_happyface_event(self, obj, event):
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
